@@ -74,16 +74,24 @@ The ``settings`` group of commands can be used to initialize a :class:`~tango.se
 file or update fields in it.
 
 """
+import logging
 import multiprocessing as mp
 import os
+import sys
 import warnings
 from pathlib import Path
-from typing import List, Optional, Sequence, Union
+from typing import TYPE_CHECKING, List, Optional, Sequence, Union
 
 import click
 from click_help_colors import HelpColorsCommand, HelpColorsGroup
 
-from tango.common.logging import click_logger, initialize_logging, teardown_logging
+from tango.common.exceptions import CliRunError
+from tango.common.logging import (
+    cli_logger,
+    initialize_logging,
+    initialize_prefix_logging,
+    teardown_logging,
+)
 from tango.common.params import Params
 from tango.common.util import (
     find_integrations,
@@ -91,7 +99,13 @@ from tango.common.util import (
     import_module_and_submodules,
 )
 from tango.settings import TangoGlobalSettings
+from tango.step_graph import StepGraph
 from tango.version import VERSION
+from tango.workspace import Workspace
+
+if TYPE_CHECKING:
+    from tango.executor import ExecutorOutput
+    from tango.workspace import Run
 
 _CLICK_GROUP_DEFAULTS = {
     "cls": HelpColorsGroup,
@@ -107,6 +121,13 @@ _CLICK_COMMAND_DEFAULTS = {
     "context_settings": {"max_content_width": 115},
 }
 
+_CALLED_BY_EXECUTOR: bool = (
+    False  # Flag used internally to determine if CLI was called by the MulticoreExecutor.
+)
+
+
+logger = logging.getLogger(__name__)
+
 
 @click.group(**_CLICK_GROUP_DEFAULTS)
 @click.version_option(version=VERSION)
@@ -132,6 +153,11 @@ _CLICK_COMMAND_DEFAULTS = {
     type=click.Choice(["fork", "spawn", "forkserver"], case_sensitive=True),
     show_choices=True,
 )
+@click.option(
+    "--called-by-executor",
+    is_flag=True,
+    hidden=True,
+)
 @click.pass_context
 def main(
     ctx,
@@ -139,7 +165,11 @@ def main(
     log_level: Optional[str] = None,
     file_friendly_logging: bool = False,
     start_method: Optional[str] = None,
+    called_by_executor: bool = False,
 ):
+    if not sys.warnoptions:
+        warnings.simplefilter("default", category=DeprecationWarning)
+
     settings: TangoGlobalSettings = (
         TangoGlobalSettings.from_file(settings)
         if settings is not None
@@ -173,11 +203,17 @@ def main(
     if file_friendly_logging:
         settings.file_friendly_logging = file_friendly_logging
 
-    initialize_logging(
-        log_level=settings.log_level,
-        file_friendly_logging=settings.file_friendly_logging,
-        enable_click_logs=True,
-    )
+    if called_by_executor:
+        # We only set this flag instead of calling `initialize_prefix_logging` here
+        # because we do not know the `step_name` yet.
+        global _CALLED_BY_EXECUTOR
+        _CALLED_BY_EXECUTOR = True
+    else:
+        initialize_logging(
+            log_level=settings.log_level,
+            file_friendly_logging=settings.file_friendly_logging,
+            enable_cli_logs=True,
+        )
 
     ctx.obj = settings
 
@@ -227,6 +263,26 @@ def cleanup(*args, **kwargs):
     help="Start a server that visualizes the current run",
     default=True,
 )
+@click.option(
+    "-j",
+    "--parallelism",
+    type=int,
+    help="The maximum number of steps to run in parallel (if possible)",
+    default=1,
+)
+@click.option(
+    "-s",
+    "--step-name",
+    help="Execute a particular step (and its dependencies) in the experiment.",
+    type=str,
+    default=None,
+)
+@click.option(
+    "-n",
+    "--name",
+    type=str,
+    help="""Specify the name for this run""",
+)
 @click.pass_obj
 def run(
     settings: TangoGlobalSettings,
@@ -236,6 +292,9 @@ def run(
     overrides: Optional[str] = None,
     include_package: Optional[Sequence[str]] = None,
     server: bool = True,
+    parallelism: int = 1,
+    step_name: Optional[str] = None,
+    name: Optional[str] = None,
 ):
     """
     Run a tango experiment.
@@ -264,6 +323,10 @@ def run(
         overrides=overrides,
         include_package=include_package,
         start_server=server,
+        parallelism=parallelism,
+        step_name=step_name,
+        name=name,
+        called_by_executor=_CALLED_BY_EXECUTOR,
     )
 
 
@@ -293,7 +356,6 @@ def server(
     Run a local webserver that watches a workspace.
     """
     from tango.server.workspace_server import WorkspaceServer
-    from tango.workspace import Workspace
     from tango.workspaces import LocalWorkspace
 
     workspace_to_watch: Workspace
@@ -314,7 +376,9 @@ def server(
         )
 
     server = WorkspaceServer.on_free_port(workspace_to_watch)
-    click_logger.info("Server started at " + click.style(server.address_for_display(), bold=True))
+    cli_logger.info(
+        "Server started at [bold underline]%s[/bold underline]", server.address_for_display()
+    )
     server.serve_forever()
 
 
@@ -326,16 +390,15 @@ def info(settings: TangoGlobalSettings):
     """
     import platform
 
-    click_logger.info(f"Tango version {VERSION} (python {platform.python_version()})")
+    cli_logger.info("Tango version %s (python %s)", VERSION, platform.python_version())
+    cli_logger.info("")
 
     # Show info about settings.
     if settings.path is not None:
-        click_logger.info("\nSettings:")
-        click_logger.info(
-            click.style(f" \N{check mark} Loaded from {str(settings.path)}", fg="green")
-        )
+        cli_logger.info("[underline]Settings:[/]")
+        cli_logger.info("[green] \N{check mark} Loaded from %s[/]", settings.path)
         if settings.include_package:
-            click_logger.info("\n   Included packages:")
+            cli_logger.info("   Included packages:")
             for package in settings.include_package:
                 is_found = True
                 try:
@@ -343,14 +406,13 @@ def info(settings: TangoGlobalSettings):
                 except (ModuleNotFoundError, ImportError):
                     is_found = False
                 if is_found:
-                    click_logger.info(click.style(f"   \N{check mark} {package}", fg="green"))
+                    cli_logger.info("   [green]\N{check mark} %s[/]", package)
                 else:
-                    click_logger.info(
-                        click.style(f"   \N{ballot x} {package} (not found)", fg="red")
-                    )
+                    cli_logger.info("   [red]\N{ballot x} %s (not found)[/]", package)
+        cli_logger.info("")
 
     # Show info about integrations.
-    click_logger.info("\nIntegrations:")
+    cli_logger.info("[underline]Integrations:[/]")
     for integration in find_integrations():
         name = integration.split(".")[-1]
         is_installed = True
@@ -359,9 +421,9 @@ def info(settings: TangoGlobalSettings):
         except (ModuleNotFoundError, ImportError):
             is_installed = False
         if is_installed:
-            click_logger.info(click.style(f" \N{check mark} {name}", fg="green"))
+            cli_logger.info(" [green]\N{check mark} %s[/]", name)
         else:
-            click_logger.info(click.style(f" \N{ballot x} {name} (not installed)", fg="yellow"))
+            cli_logger.info(" [yellow]\N{ballot x} %s (not installed)[/]", name)
 
 
 @main.group(**_CLICK_GROUP_DEFAULTS)
@@ -395,9 +457,8 @@ def init(settings: TangoGlobalSettings, path: Optional[str] = None, force: bool
     if path_to_write.is_file() and not force:
         raise click.ClickException("Settings file already exists! Use -f/--force to overwrite it.")
     settings.to_file(path_to_write)
-    click_logger.info(
-        click.style("\N{check mark} Settings file written to ", fg="green")
-        + click.style(path_to_write, fg="green", bold=True)
+    cli_logger.info(
+        "[green]\N{check mark} Settings file written to [bold]%s[/bold][/green]", path_to_write
     )
 
 
@@ -444,8 +505,6 @@ def workspace(
         settings.workspace = {"type": "from_url", "url": workspace}
 
     if validate:
-        from tango.workspace import Workspace
-
         for package_name in settings.include_package or []:
             import_extra_module(package_name)
 
@@ -582,12 +641,16 @@ def _run(
     overrides: Optional[str] = None,
     include_package: Optional[Sequence[str]] = None,
     start_server: bool = True,
+    step_name: Optional[str] = None,
+    parallelism: int = 1,
+    multicore: Optional[bool] = None,
+    name: Optional[str] = None,
+    called_by_executor: bool = False,
 ) -> str:
     from tango.executor import Executor
+    from tango.executors import MulticoreExecutor
     from tango.server.workspace_server import WorkspaceServer
-    from tango.step_graph import StepGraph
-    from tango.workspace import Workspace
-    from tango.workspaces import default_workspace
+    from tango.workspaces import MemoryWorkspace, default_workspace
 
     # Read params.
     params = Params.from_file(experiment, params_overrides=overrides or "")
@@ -611,48 +674,163 @@ def _run(
     else:
         workspace = default_workspace
 
-    # Initialize step graph and register run.
-    step_graph = StepGraph(params.pop("steps", keep_as_dict=True))
-    params.assert_empty("'tango run'")
-    run = workspace.register_run(step for step in step_graph.values())
+    if multicore is None:
+        if isinstance(workspace, MemoryWorkspace):
+            # Memory workspace does not work with multiple cores.
+            multicore = False
+        elif "pydevd" in sys.modules:
+            # Pydevd doesn't reliably follow child processes, so we disable multicore under the debugger.
+            logger.warning("Debugger detected, disabling multicore.")
+            multicore = False
+        else:
+            multicore = True
 
-    # Capture logs to file.
-    with workspace.capture_logs_for_run(run.name):
-        click_logger.info(
-            click.style("Starting new run ", fg="green")
-            + click.style(run.name, fg="green", bold=True)
+    # Initialize step graph and register run.
+    step_graph = StepGraph.from_params(params.pop("steps", keep_as_dict=True))
+    params.assert_empty("'tango run'")
+
+    if step_name is not None:
+        assert step_name in step_graph, (
+            f"You want to run a step called '{step_name}', but it cannot be found in the experiment config. "
+            f"The config contains: {list(step_graph.keys())}."
         )
 
+        if called_by_executor and name is not None:
+            try:
+                run = workspace.registered_run(name)
+            except KeyError:
+                raise RuntimeError(
+                    "The CLI was called by `MulticoreExecutor.execute_step_graph`, but "
+                    f"'{name}' is not already registered as a run. This should never happen!"
+                )
+        else:
+            sub_graph = step_graph.sub_graph(step_name)
+            run = workspace.register_run((step for step in sub_graph.values()), name)
+    else:
+        run = workspace.register_run((step for step in step_graph.values()), name)
+
+    def log_and_execute_run():
+        if step_name is None:
+            cli_logger.info("[green]Starting new run [bold]%s[/][/]", run.name)
+
         # Initialize server.
         if start_server:
             server = WorkspaceServer.on_free_port(workspace)
             server.serve_in_background()
-            click_logger.info(
-                "Server started at " + click.style(server.address_for_display(run.name), bold=True)
+            cli_logger.info(
+                "Server started at [bold]%s[/bold]", server.address_for_display(run.name)
             )
 
+        executor: Executor
         # Initialize Executor and execute the step graph.
-        executor = Executor(workspace=workspace, include_package=include_package)
-        executor.execute_step_graph(step_graph)
+        if multicore:
+            executor = MulticoreExecutor(
+                workspace=workspace, include_package=include_package, parallelism=parallelism
+            )
+        else:
+            executor = Executor(workspace=workspace, include_package=include_package)
 
-        # Print everything that has been computed.
-        ordered_steps = sorted(step_graph.values(), key=lambda step: step.name)
-        for step in ordered_steps:
-            info = workspace.step_info(step)
-            if info.result_location is not None:
-                click_logger.info(
-                    click.style("\N{check mark} The output for ", fg="green")
-                    + click.style(f'"{step.name}"', bold=True, fg="green")
-                    + click.style(" is in ", fg="green")
-                    + click.style(f"{info.result_location}", bold=True, fg="green")
+        if step_name is not None:
+            step = step_graph[step_name]
+            executor.execute_step(step)
+            if not called_by_executor:
+                cli_logger.info(
+                    "[green]\N{check mark} Finished run for step [bold]%s[/] (%s)[/]",
+                    step_name,
+                    run.name,
                 )
+        else:
+            executor_output = executor.execute_step_graph(step_graph, run_name=run.name)
+            if executor_output.failed:
+                cli_logger.error(
+                    "[red]\N{ballot x} Run [bold]%s[/] finished with errors[/]", run.name
+                )
+            elif not called_by_executor:
+                cli_logger.info("[green]\N{check mark} Finished run [bold]%s[/][/]", run.name)
+            _display_run_results(run, step_graph, workspace, executor_output)
 
-        click_logger.info(
-            click.style("Finished run ", fg="green") + click.style(run.name, fg="green", bold=True)
-        )
+            if executor_output.failed:
+                raise CliRunError
+
+    if called_by_executor:
+        from tango.common.aliases import EnvVarNames
+
+        # We set this environment variable so that any steps that contain multiprocessing
+        # and call `initialize_worker_logging` also log the messages with the `step_name` prefix.
+        os.environ[EnvVarNames.LOGGING_PREFIX.value] = f"step {step_name}"
+        initialize_prefix_logging(f"step {step_name}", main_process=False)
+        log_and_execute_run()
+    else:
+        # Capture logs to file.
+        with workspace.capture_logs_for_run(run.name):
+            log_and_execute_run()
 
     return run.name
 
 
+def _display_run_results(
+    run: "Run", step_graph: StepGraph, workspace: Workspace, executor_output: "ExecutorOutput"
+) -> None:
+    from rich.table import Table
+
+    table = Table(caption_style="")
+    table.add_column("Step Name", justify="left", style="cyan")
+    table.add_column("Status", justify="left")
+    table.add_column("Cached Result", justify="left")
+    last_cached_step: Optional[str] = None
+    for step in sorted(step_graph.values(), key=lambda step: step.name):
+        step_info = workspace.step_info(step)
+
+        status_str: str
+        if step.name in executor_output.failed:
+            status_str = "[red]\N{ballot x} failed[/]"
+        elif step.name in executor_output.not_run:
+            status_str = "[yellow]- not run[/]"
+        elif step.name in executor_output.successful:
+            status_str = "[green]\N{check mark} succeeded[/]"
+        else:
+            continue
+
+        result_str: str
+        if step_info.result_location is not None:
+            last_cached_step = step.name
+            result_str = f"[cyan]{step_info.result_location}[/]"
+        else:
+            result_str = "[grey62]N/A[/]"
+
+        table.add_row(step.name, status_str, result_str)
+
+    caption_parts: List[str] = []
+    if executor_output.failed:
+        caption_parts.append(
+            f"[red]\N{ballot x}[/] [italic]{len(executor_output.failed)} failed[/]"
+        )
+    if executor_output.successful:
+        caption_parts.append(
+            f"[green]\N{check mark}[/] [italic]{len(executor_output.successful)} succeeded[/]"
+        )
+    if executor_output.not_run:
+        caption_parts.append(f"[italic]{len(executor_output.not_run)} not run[/]")
+    table.caption = ", ".join(caption_parts)
+
+    cli_logger.info(table)
+
+    if not executor_output.failed and last_cached_step is not None:
+        from rich.syntax import Syntax
+
+        example = Syntax(
+            "\n".join(
+                [
+                    ">>> from tango import Workspace",
+                    ">>> workspace = Workspace.from_url(...)",
+                    f'>>> workspace.step_result_for_run("{run.name}", "{last_cached_step}")',
+                ]
+            ),
+            "python",
+        )
+        cli_logger.info("Use your workspace to get the cached result of a step, e.g.")
+        cli_logger.info(example)
+
+
 if __name__ == "__main__":
     main()
