@@ -5,13 +5,26 @@ import string
 import sys
 import traceback
 from contextlib import contextmanager
+from datetime import datetime, tzinfo
 from pathlib import Path
 from typing import Iterable, Optional, Set, Tuple, Union
 
+import pytz
+
 from .aliases import PathOrStr
 from .exceptions import SigTermReceived
 
 
+def tango_cache_dir() -> Path:
+    """
+    Returns a directory suitable for caching things from Tango, defaulting
+    to ``$HOME/.cache/tango``.
+    """
+    cache_dir = Path.home() / ".cache" / "tango"
+    cache_dir.mkdir(parents=True, exist_ok=True)
+    return cache_dir
+
+
 def _handle_sigterm(sig, frame):
     raise SigTermReceived
 
@@ -194,7 +207,7 @@ def could_be_class_name(name: str) -> bool:
 
 
 def _is_valid_python_name(name: str) -> bool:
-    return bool(name and name[0].isalpha() and name.isalnum())
+    return bool(name and name[0].isalpha() and name.replace("_", "").isalnum())
 
 
 def threaded_generator(g, queue_size: int = 16):
@@ -243,3 +256,11 @@ def exception_to_string(e: BaseException) -> str:
     else:
         formatted = traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__)
     return "".join(formatted)
+
+
+def utc_now_datetime() -> datetime:
+    return datetime.utcnow().replace(tzinfo=pytz.utc)
+
+
+def local_timezone() -> Optional[tzinfo]:
+    return datetime.now().astimezone().tzinfo
