@@ -1,4 +1,5 @@
 import logging
+import os
 import shutil
 from itertools import islice
 from typing import Any, Dict, List, Optional, Set, cast
@@ -16,6 +17,7 @@ from tango.common.tqdm import Tqdm
 from tango.common.util import get_extra_imported_modules, import_extra_module
 from tango.format import Format
 from tango.step import Step
+from tango.workspace import Workspace
 
 from .data import DataLoader
 from .exceptions import StopEarly
@@ -172,6 +174,38 @@ class TorchTrainStep(Step):
             The trained model on CPU with the weights from the best checkpoint loaded.
 
         """
+
+        devices = self._get_devices(device_count)
+
+        return self._train(
+            model=model,
+            training_engine=training_engine,
+            dataset_dict=dataset_dict,
+            train_dataloader=train_dataloader,
+            train_split=train_split,
+            validation_split=validation_split,
+            validation_dataloader=validation_dataloader,
+            seed=seed,
+            train_steps=train_steps,
+            train_epochs=train_epochs,
+            validation_steps=validation_steps,
+            grad_accum=grad_accum,
+            log_every=log_every,
+            checkpoint_every=checkpoint_every,
+            validate_every=validate_every,
+            devices=devices,
+            distributed_port=distributed_port,
+            val_metric_name=val_metric_name,
+            minimize_val_metric=minimize_val_metric,
+            auto_aggregate_val_metric=auto_aggregate_val_metric,
+            callbacks=callbacks,
+            remove_stale_checkpoints=remove_stale_checkpoints,
+        )
+
+    def _get_devices(self, device_count: int) -> List[int]:
+        """
+        Validates the device count, and returns the list of devices.
+        """
         # Validate device(s).
         if device_count <= 0:
             raise ConfigurationError("Invalid value for 'device_count'. Must be at least 1.")
@@ -184,12 +218,34 @@ class TorchTrainStep(Step):
             self.logger.info(
                 "Training on CPU with %d worker%s", device_count, "s" if device_count > 1 else ""
             )
+        return devices
 
-        if validate_every is not None and validation_split is None:
-            raise ConfigurationError(
-                "You have set a validation interval, but no validation split. "
-                "That's probably unintentional."
-            )
+    def _train(
+        self,
+        model: Lazy[Model],
+        training_engine: Lazy[TrainingEngine],
+        dataset_dict: DatasetDictBase,
+        train_dataloader: Lazy[DataLoader],
+        *,
+        train_split: str = "train",
+        validation_split: Optional[str] = None,
+        validation_dataloader: Optional[Lazy[DataLoader]] = None,
+        seed: int = 42,
+        train_steps: Optional[int] = None,
+        train_epochs: Optional[int] = None,
+        validation_steps: Optional[int] = None,
+        grad_accum: int = 1,
+        log_every: int = 10,
+        checkpoint_every: int = 100,
+        validate_every: Optional[int] = None,
+        devices: Optional[List[int]] = None,
+        distributed_port: int = 54761,
+        val_metric_name: str = "loss",
+        minimize_val_metric: bool = True,
+        auto_aggregate_val_metric: bool = True,
+        callbacks: Optional[List[Lazy[TrainCallback]]] = None,
+        remove_stale_checkpoints: bool = True,
+    ) -> Model:
 
         is_distributed = False
         num_workers = 1
@@ -197,6 +253,12 @@ class TorchTrainStep(Step):
             is_distributed = True
             num_workers = len(devices)
 
+        if validate_every is not None and validation_split is None:
+            raise ConfigurationError(
+                "You have set a validation interval, but no validation split. "
+                "That's probably unintentional."
+            )
+
         if (train_steps is not None) == (train_epochs is not None):
             raise ConfigurationError(
                 "One of 'train_steps' or 'train_epochs' needs to be specified, but not both."
@@ -205,6 +267,7 @@ class TorchTrainStep(Step):
         config = TrainConfig(
             self.unique_id,
             self.work_dir,
+            step_name=self.name,
             train_split=train_split,
             validation_split=validation_split,
             seed=seed,
@@ -232,6 +295,7 @@ class TorchTrainStep(Step):
             mp.spawn(
                 _train,
                 args=(
+                    self.workspace,
                     config,
                     model,
                     training_engine,
@@ -248,6 +312,7 @@ class TorchTrainStep(Step):
         else:
             final_model = _train(  # type: ignore[assignment]
                 0,
+                self.workspace,
                 config,
                 model,
                 training_engine,
@@ -273,6 +338,7 @@ class TorchTrainStep(Step):
 
 def _train(
     worker_id: int,
+    workspace: Workspace,
     config: TrainConfig,
     model: Lazy[Model],
     training_engine: Lazy[TrainingEngine],
@@ -365,6 +431,7 @@ def _train(
     # Initialize callbacks.
     callbacks: List[TrainCallback] = [
         callback.construct(
+            workspace=workspace,
             train_config=config,
             training_engine=training_engine,
             dataset_dict=dataset_dict,
@@ -666,6 +733,10 @@ def _train(
         callback.post_train_loop(step, current_epoch)
 
     if config.is_local_main_process:
+        # It's possible this file already exists if the step previously failed after
+        # already saving the final weights.
+        if config.final_weights_path.is_file():
+            os.remove(config.final_weights_path)
         training_engine.save_complete_weights_from_checkpoint(
             config.best_state_path, config.final_weights_path
         )
