@@ -66,6 +66,10 @@ library under the corresponding class from the `torch <torch.html>`_ integration
             transformers::AutoModelForImageClassification::from_pretrained
             transformers::AutoModelForImageSegmentation::from_config
             transformers::AutoModelForImageSegmentation::from_pretrained
+            transformers::AutoModelForInstanceSegmentation::from_config
+            transformers::AutoModelForInstanceSegmentation::from_pretrained
+            transformers::AutoModelForMaskedImageModeling::from_config
+            transformers::AutoModelForMaskedImageModeling::from_pretrained
             transformers::AutoModelForMaskedLM::from_config
             transformers::AutoModelForMaskedLM::from_pretrained
             transformers::AutoModelForMultipleChoice::from_config
@@ -78,6 +82,8 @@ library under the corresponding class from the `torch <torch.html>`_ integration
             transformers::AutoModelForPreTraining::from_pretrained
             transformers::AutoModelForQuestionAnswering::from_config
             transformers::AutoModelForQuestionAnswering::from_pretrained
+            transformers::AutoModelForSemanticSegmentation::from_config
+            transformers::AutoModelForSemanticSegmentation::from_pretrained
             transformers::AutoModelForSeq2SeqLM::from_config
             transformers::AutoModelForSeq2SeqLM::from_pretrained
             transformers::AutoModelForSequenceClassification::from_config
@@ -180,10 +186,19 @@ library under the corresponding class from the `torch <torch.html>`_ integration
             transformers::DefaultDataCollator
 """
 
-__all__ = ["RunGeneration", "RunGenerationDataset", "Tokenizer", "Config"]
+__all__ = [
+    "RunGeneration",
+    "RunGenerationDataset",
+    "Tokenizer",
+    "Config",
+    "FinetuneWrapper",
+    "FinetuneStep",
+    "TokenizeText2TextData",
+]
 
 from .config import Config
 from .data import *  # noqa: F403
+from .finetune import FinetuneStep, FinetuneWrapper, TokenizeText2TextData
 from .model import *  # noqa: F403
 from .optim import *  # noqa: F403
 from .run_generation import RunGeneration, RunGenerationDataset
