@@ -23,7 +23,18 @@ from typing import (
     cast,
 )
 
-import click
+from tango.common.det_hash import CustomDetHash, det_hash
+from tango.common.exceptions import ConfigurationError
+from tango.common.from_params import (
+    infer_constructor_params,
+    infer_method_params,
+    pop_and_construct_arg,
+)
+from tango.common.lazy import Lazy
+from tango.common.logging import cli_logger
+from tango.common.params import Params
+from tango.common.registrable import Registrable
+from tango.format import DillFormat, Format
 
 try:
     from typing import get_args, get_origin  # type: ignore
@@ -36,19 +47,6 @@ except ImportError:
         return getattr(tp, "__args__", ())
 
 
-from tango.common.det_hash import CustomDetHash, det_hash
-from tango.common.exceptions import ConfigurationError
-from tango.common.from_params import (
-    infer_constructor_params,
-    infer_method_params,
-    pop_and_construct_arg,
-)
-from tango.common.lazy import Lazy
-from tango.common.logging import click_logger
-from tango.common.params import Params
-from tango.common.registrable import Registrable
-from tango.format import DillFormat, Format
-
 if TYPE_CHECKING:
     from tango.workspace import Workspace
 
@@ -79,6 +77,14 @@ class Step(Registrable, Generic[T]):
     :param step_format: gives you a way to override the step's default format (which is given in :attr:`FORMAT`).
     :param step_config: is the original raw part of the experiment config corresponding to this step.
       This can be accessed via the :attr:`config` property within each step's :meth:`run()` method.
+    :param step_unique_id_override: overrides the construction of the step's unique id using the hash
+      of inputs.
+
+    .. important::
+        Overriding the unique id means that the step will always map to this value, regardless of the inputs,
+        and therefore, the step cache will only hold a single copy of the step's output (from the last execution).
+        Thus, in most cases, this should not be used when constructing steps. We include this option for the case
+        when the executor creates subprocesses, which also need to access the *same* ``Step`` object.
     """
 
     DETERMINISTIC: bool = True
@@ -111,12 +117,18 @@ class Step(Registrable, Generic[T]):
     the model output, not about how many outputs you can produce at the same time.
     """
 
+    _UNIQUE_ID_SUFFIX: Optional[str] = None
+    """
+    Used internally for testing.
+    """
+
     def __init__(
         self,
         step_name: Optional[str] = None,
         cache_results: Optional[bool] = None,
         step_format: Optional[Format] = None,
         step_config: Optional[Dict[str, Any]] = None,
+        step_unique_id_override: Optional[str] = None,
         **kwargs,
     ):
         if self.VERSION is not None:
@@ -138,7 +150,7 @@ class Step(Registrable, Generic[T]):
         else:
             self.format = step_format
 
-        self.unique_id_cache: Optional[str] = None
+        self.unique_id_cache = step_unique_id_override
         if step_name is None:
             self.name = self.unique_id
         else:
@@ -185,6 +197,7 @@ class Step(Registrable, Generic[T]):
                 f"Step {self.name}'s cache_results parameter is set to an invalid value."
             )
 
+        self._workspace: Optional["Workspace"] = None
         self.work_dir_for_run: Optional[
             Path
         ] = None  # This is set only while the run() method runs.
@@ -329,13 +342,15 @@ class Step(Registrable, Generic[T]):
         """
         raise NotImplementedError()
 
-    def _run_with_work_dir(self, workspace: "Workspace", **kwargs) -> T:
+    def _run_with_work_dir(self, workspace: "Workspace", needed_by: Optional["Step"] = None) -> T:
         if self.work_dir_for_run is not None:
             raise RuntimeError("You can only run a Step's run() method once at a time.")
 
         if self.DETERMINISTIC:
             random.seed(784507111)
 
+        self._workspace = workspace
+
         if self.cache_results:
             self.work_dir_for_run = workspace.work_dir(self)
             dir_for_cleanup = None
@@ -344,20 +359,32 @@ class Step(Registrable, Generic[T]):
             self.work_dir_for_run = Path(dir_for_cleanup.name)
 
         try:
-            if self.cache_results:
-                workspace.step_starting(self)
+            kwargs = self._replace_steps_with_results(self.kwargs, workspace)
+
+            if needed_by:
+                cli_logger.info(
+                    '[blue]\N{black circle} Starting step [bold]"%s"[/] (needed by "%s")...[/]',
+                    self.name,
+                    needed_by.name,
+                )
+            else:
+                cli_logger.info(
+                    '[blue]\N{black circle} Starting step [bold]"%s"[/]...[/]',
+                    self.name,
+                )
+
+            workspace.step_starting(self)
+
             try:
                 result = self.run(**kwargs)
-                if self.cache_results:
-                    result = workspace.step_finished(self, result)
+                result = workspace.step_finished(self, result)
+                cli_logger.info(f'[green]\N{check mark} Finished step [bold]"{self.name}"[/][/]')
                 return result
             except BaseException as e:
-                # TODO (epwalsh): do we want to handle KeyboardInterrupts differently?
-                # Maybe have a `workspace.step_interrupted()` method?
-                if self.cache_results:
-                    workspace.step_failed(self, e)
+                workspace.step_failed(self, e)
                 raise
         finally:
+            self._workspace = None
             self.work_dir_for_run = None
             if dir_for_cleanup is not None:
                 dir_for_cleanup.cleanup()
@@ -379,6 +406,20 @@ class Step(Registrable, Generic[T]):
             )
         return self.work_dir_for_run
 
+    @property
+    def workspace(self) -> "Workspace":
+        """
+        The :class:`~tango.workspace.Workspace` being used.
+
+        This is a convenience property for you to call inside your :meth:`run()` method.
+        """
+        if self._workspace is None:
+            raise RuntimeError(
+                "You can only call this method while the step is running with a workspace. "
+                "Did you call '.run()' directly? You should only run a step with '.result()'."
+            )
+        return self._workspace
+
     @property
     def config(self) -> Dict[str, Any]:
         """
@@ -421,8 +462,10 @@ class Step(Registrable, Generic[T]):
                 )[:32]
             else:
                 self.unique_id_cache += det_hash(
-                    _random_for_step_names.getrandbits((58 ** 32).bit_length())
+                    _random_for_step_names.getrandbits((58**32).bit_length())
                 )[:32]
+            if self._UNIQUE_ID_SUFFIX is not None:
+                self.unique_id_cache += f"-{self._UNIQUE_ID_SUFFIX}"
 
         return self.unique_id_cache
 
@@ -482,33 +525,21 @@ class Step(Registrable, Generic[T]):
             workspace = default_workspace
 
         if self.cache_results and self in workspace.step_cache:
-            if click_logger.isEnabledFor(logging.INFO):
-                message = click.style("\N{check mark} Found output for step ", fg="green")
-                message += click.style(f'"{self.name}"', bold=True, fg="green")
-                message += click.style(" in cache", fg="green")
-                if needed_by is None:
-                    message += click.style(" ...", fg="green")
-                else:
-                    message += click.style(f' (needed by "{needed_by.name}") ...', fg="green")
-                click_logger.info(message)
-            return workspace.step_cache[self]
-
-        kwargs = self._replace_steps_with_results(self.kwargs, workspace)
-
-        if click_logger.isEnabledFor(logging.INFO):
-            message = click.style("\N{black circle} Starting step ", fg="blue")
-            message += click.style(f'"{self.name}"', bold=True, fg="blue")
-            if needed_by is None:
-                message += click.style(" ...", fg="blue")
+            if needed_by:
+                cli_logger.info(
+                    '[green]\N{check mark} Found output for step [bold]"%s"[/bold] in cache '
+                    '(needed by "%s")...[/green]',
+                    self.name,
+                    needed_by.name,
+                )
             else:
-                message += click.style(f' (needed by "{needed_by.name}") ...', fg="blue")
-            click_logger.info(message)
-        result = self._run_with_work_dir(workspace, **kwargs)
-        click_logger.info(
-            click.style("\N{check mark} Finished step ", fg="green")
-            + click.style(f'"{self.name}"', bold=True, fg="green")
-        )
-        return result
+                cli_logger.info(
+                    '[green]\N{check mark} Found output for step [bold]"%s"[/] in cache...[/]',
+                    self.name,
+                )
+            return workspace.step_cache[self]
+        else:
+            return self._run_with_work_dir(workspace, needed_by=needed_by)
 
     def ensure_result(
         self,
@@ -534,7 +565,7 @@ class Step(Registrable, Generic[T]):
                 yield from dependencies_internal(o.kwargs)
             elif isinstance(o, str):
                 return  # Confusingly, str is an Iterable of itself, resulting in infinite recursion.
-            elif isinstance(o, dict):
+            elif isinstance(o, (dict, Params)):
                 yield from dependencies_internal(o.values())
             elif isinstance(o, Iterable):
                 yield from itertools.chain(*(dependencies_internal(i) for i in o))
